<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Controller;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class CommandController extends \TYPO3\CMS\Extbase\Mvc\Controller\CommandController
{
	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 */
	protected $configurationManager;

	/**
	 * @param \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager
	 */
	public function injectConfigurationManager(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager)
    {
		$this->configurationManager = $configurationManager;
		$this->settings = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
	}

    /**
     * @param integer $pageUid
     */
    protected function initFrontend($pageUid)
    {
        if (!$this->configurationManager->getContentObject()) {
            $this->configurationManager->setContentObject(
                $this->objectManager->create('TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer')
            );
        }

        if (!$GLOBALS['TSFE'] || $GLOBALS['TSFE']->id != $pageUid) {
            $rootline = \TYPO3\CMS\Backend\Utility\BackendUtility::BEgetRootLine($pageUid);
            $host = \TYPO3\CMS\Backend\Utility\BackendUtility::firstDomainRecord($rootline);

            $_SERVER['HTTP_HOST'] = $host;
            $_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'] = 'typo3';

            if (!is_object($GLOBALS['TT'])) {
                $GLOBALS['TT'] = $this->objectManager->create('TYPO3\CMS\Core\TimeTracker\TimeTracker');
                $GLOBALS['TT']->start();
            }

            if (!$GLOBALS['TSFE']) {
                $GLOBALS['TSFE'] = new \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController(
                    $GLOBALS['TYPO3_CONF_VARS'], $pageUid, '0');
            }

            $GLOBALS['TSFE']->connectToDB();
            $GLOBALS['TSFE']->initFEuser();
            $GLOBALS['TSFE']->fetch_the_id();
            $GLOBALS['TSFE']->determineId();
            $GLOBALS['TSFE']->getPageAndRootline();
            $GLOBALS['TSFE']->initTemplate();
            $GLOBALS['TSFE']->tmpl->getFileName_backPath = PATH_site;
            $GLOBALS['TSFE']->forceTemplateParsing = 1;
            $GLOBALS['TSFE']->getConfigArray();

            $this->uriBuilder = $this->objectManager->create('TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder');
            $this->uriBuilder->setRequest($this->objectManager->create('TYPO3\CMS\Extbase\Mvc\Web\Request'));
            $this->uriBuilder->initializeObject();
        }
    }

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    protected function uriFor($targetPageUid, $actionName = null, $controllerArguments = array(), $controllerName = null, $extensionName = null, $pluginName = null, $format = '', $argumentPrefix = null)
    {
        if ($actionName !== null) {
            $controllerArguments['action'] = $actionName;
        }
        if ($controllerName !== null) {
            $controllerArguments['controller'] = $controllerName;
        } else {
            $controllerArguments['controller'] = $this->request->getControllerName();
        }
        if ($extensionName === null) {
            $extensionName = $this->request->getControllerExtensionName();
        }
        if ($pluginName === null && TYPO3_MODE === 'FE') {
            $pluginName = \Tx_Extbase_Utility_Extension::getPluginNameByAction($extensionName, $controllerArguments['controller'], $controllerArguments['action']);
        }
        if ($pluginName === null) {
            $pluginName = $this->request->getPluginName();
        }
        if ($targetPageUid === null && TYPO3_MODE === 'FE') {
            $targetPageUid = \Tx_Extbase_Utility_Extension::getTargetPidByPlugin($extensionName, $pluginName);
        }
        if ($format !== '') {
            $controllerArguments['format'] = $format;
        }
        if ($argumentPrefix !== null) {
            $prefixedControllerArguments = array($argumentPrefix => $controllerArguments);
        } else {
            $pluginNamespace = $this->getPluginNamespace($pluginName, $extensionName);
            $prefixedControllerArguments = array($pluginNamespace => $controllerArguments);
        }

        return $prefixedControllerArguments;
    }

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param boolean $noCache
     * @param boolean $useCacheHash
     * @param boolean $createAbsoluteUri
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    protected function buildFrontendUri($targetPageUid, $actionName = null, $controllerArguments = array(),
        $controllerName = null, $extensionName = null, $pluginName = null, $noCache = false, $useCacheHash = true,
        $createAbsoluteUri = true, $format = '', $argumentPrefix = null)
    {
        $this->initFrontend($targetPageUid);

        $this->uriBuilder->reset();
        $this->uriBuilder
            ->setNoCache($noCache)
            ->setUseCacheHash($useCacheHash)
            ->setCreateAbsoluteUri($createAbsoluteUri)
            ->setArguments($this->uriFor(
                $targetPageUid,
                $actionName,
                $controllerArguments,
                $controllerName,
                $extensionName,
                $pluginName,
                $format,
                $argumentPrefix
            ))
        ;

        return str_replace(PATH_site, '/', $this->uriBuilder->buildFrontendUri());
    }

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    protected function translate($key, $arguments = array())
    {
        return (string)\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * @param type $languageKey
     */
    protected function setLanguage($languageKey)
    {
        if (!$GLOBALS['BE_USER']) {
            $GLOBALS['BE_USER'] = new \stdClass();
            $GLOBALS['BE_USER']->uc = array('lang' => $languageKey);
        }

        $GLOBALS['BE_USER']->uc['lang'] = $languageKey;
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
                $extensionName ?: $this->extensionName,
                $pluginName ?: $this->request->getPluginName()
        );

        return $pluginNamespace;
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    protected function sendMail(array $recipient, array $sender, $subject, $body, $format = 'plain', $closure = null)
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        switch ($format) {
            case 'html':
                break;
            case 'plain':
            default:
                $format = 'plain';
                break;
        }

        foreach ($recipient as $key => $value) {
            if (!\TYPO3\CMS\Core\Utility\GeneralUtility::validEmail($key)) {
                if (!\TYPO3\CMS\Core\Utility\GeneralUtility::validEmail($value)) {
                    unset($recipient[$key]);
                }
            }
        }
        foreach ($sender as $key => $value) {
            if (!\TYPO3\CMS\Core\Utility\GeneralUtility::validEmail($key)) {
                if (!\TYPO3\CMS\Core\Utility\GeneralUtility::validEmail($value)) {
                    unset($sender[$key]);
                }
            }
        }
        
        if (count($recipient) > 0 && count($sender) > 0) {
            $message->setTo($recipient)
                ->setFrom($sender)
                ->setSubject($subject)
                ->setBody($body, 'text/' . $format);

            if ($closure) {
                try {
                    $closure($message);
                } catch(\Exception $e) {}
            }

            $message->send();

            return $message->isSent();
        } else {
            return false;
        }
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    protected function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
        $format = 'plain', $closure = null)
    {
        switch ($format) {
            case 'html':
                $ext = 'html';
                break;
            case 'text':
            case 'plain':
            default:
                $ext = 'txt';
                break;
        }

        $message = $this->renderView('Email', $templateName, $arguments, $ext);

        return $this->sendMail(
            $recipient,
            $sender,
            $subject,
            $message,
            $format,
            $closure
        );
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    protected function renderView($controller, $action, $arguments, $ext = 'html')
    {
        $view = $this->objectManager->create('TYPO3\CMS\Fluid\View\StandaloneView');
        $view->setFormat($ext);

        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['templateRootPath']);

        if (!$templateRootPath) {
            $templateRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Templates/'
            );
        }

        $partialRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['partialRootPath']);

        if (!$partialRootPath) {
            $partialRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Partials/'
            );
        }
        $view->setPartialRootPath($partialRootPath);

        $layoutRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['layoutRootPath']);

        if (!$layoutRootPath) {
            $layoutRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Layouts/'
            );
        }
        $view->setLayoutRootPath($layoutRootPath);

        $templatePathAndFilename = $templateRootPath . $controller . '/' . $action . '.' . $ext;
        $view->setTemplatePathAndFilename($templatePathAndFilename);

        if ($arguments['settings'] && is_array($arguments['settings'])) {
            if ($this->settings && is_array($this->settings)) {
                $arguments['settings'] = \TYPO3\CMS\Core\Utility\GeneralUtility::array_merge_recursive_overrule($this->settings, $arguments['settings']);
            }
        } else {
            $arguments['settings'] = $this->settings;
        }

        $view->assignMultiple($arguments);

        return $view->render();
    }

    /**
     * Function: sanitize
     * Returns a sanitized string, typically for URLs.
     *
     * Parameters:
     *     $string - The string to sanitize.
     *     $forceLowercase - Force the string to lowercase?
     *     $anal - If set to *true*, will remove all non-alphanumeric characters.
     */
    protected function sanitize($string, $forceLowercase = true, $anal = false)
    {
        $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
                       "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
                       "â€”", "â€“", ",", "<", ".", ">", "/", "?");

        $clean = trim(str_replace($strip, "", strip_tags($string)));
        $clean = preg_replace('/\s+/', "-", $clean);
        $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;

        return ($forceLowercase) ?
            (function_exists('mb_strtolower')) ?
                mb_strtolower($clean, 'UTF-8') :
                strtolower($clean) :
            $clean;
    }
}