<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Metadata;

/**
 * @package     imia_base_ext
 * @subpackage  Metadata
 * @author      David Frerich <d.frerich@imia.de>
 */
class ClassMetadata extends \Metadata\MergeableClassMetadata
{
    /**
     * @var string
     */
    public $mergeType;

    /**
     * @var \IMIA\BaseExt\Annotation\SQL\Table
     */
    public $sqlTable;

    /**
     * @var \IMIA\BaseExt\Annotation\TCA\Table
     */
    public $tcaTable;

    /**
     * @var \IMIA\BaseExt\Annotation\TCA\AddPaletteToTypes
     */
    public $tcaAddPaletteToTypes;

    /**
     * @var \IMIA\BaseExt\Annotation\TCA\AddPalettesToTypes
     */
    public $tcaAddPalettesToTypes;

    /**
     * @param \Metadata\MergeableInterface $object
     */
    public function merge(\Metadata\MergeableInterface $object)
    {

        $tcaProperties = array('tcaTable', 'tcaAddPaletteToTypes', 'tcaAddPalettesToTypes');
        $sqlProperties = array('sqlTable');

        $currentExtKey = \IMIA\BaseExt\Utility\DynamicConfiguration::getExtKeyForClass($this->name);
        $nextExtKey = \IMIA\BaseExt\Utility\DynamicConfiguration::getExtKeyForClass($object->name);

        if ($currentExtKey && $nextExtKey && $currentExtKey !== $nextExtKey) {
            // conflicts detected
            switch ($this->mergeType) {
                case 'sql':
                    // merge sql (table name), don't merge tca
                    foreach ($sqlProperties as $property) {
                        $this->mergeObjectProperty($object, $property);
                    }
                    foreach ($tcaProperties as $property) {
                        $this->$property = $object->$property;
                    }

                    // replace property metadata
                    foreach ($this->propertyMetadata as $field => $propertyMetaData) {
                        if ($propertyMetaData) {
                            $this->propertyMetadata[$field] = $object->propertyMetadata[$field]
                                ? clone $object->propertyMetadata[$field]
                                : null;
                        }
                    }
                    break;
                case 'tca':
                    // merge sql (table name), don't merge tca
                    foreach ($sqlProperties as $property) {
                        $this->mergeObjectProperty($object, $property);
                    }
                    foreach ($tcaProperties as $property) {
                        $this->$property = $object->$property;
                    }

                    // replace property metadata
                    foreach ($this->propertyMetadata as $field => $propertyMetaData) {
                        if ($propertyMetaData && is_object($propertyMetaData)) {
                            $sqlColumn = $propertyMetaData->sqlColumn
                                ? clone $propertyMetaData->sqlColumn
                                : null;

                            $this->propertyMetadata[$field] = $object->propertyMetadata[$field]
                                ? clone $object->propertyMetadata[$field]
                                : null;

                            if ($this->propertyMetadata[$field] && is_object($this->propertyMetadata[$field])) {
                                $this->propertyMetadata[$field]->sqlColumn = $sqlColumn;
                            }
                        }
                    }
                    break;
                case 'ts':
                    // merge sql and tca
                    foreach ($sqlProperties as $property) {
                        $this->mergeObjectProperty($object, $property);
                    }
                    foreach ($tcaProperties as $property) {
                        $this->mergeObjectProperty($object, $property);
                    }
                    break;
            }
        } else {
            // merge all, no problems
            foreach ($sqlProperties as $property) {
                $this->mergeObjectProperty($object, $property);
            }
            foreach ($tcaProperties as $property) {
                $this->mergeObjectProperty($object, $property);
            }
        }

        parent::merge($object);
    }

    /**
     * @param \Metadata\MergeableInterface $object
     * @param string $property
     */
    private function mergeObjectProperty(\Metadata\MergeableInterface $object, $property)
    {
        if ($this->$property && $object->$property) {
            foreach (get_object_vars($this->$property) as $objectProperty => $value) {
                if ($object->$property->$objectProperty) {
                    $this->$property->$objectProperty = $object->$property->$objectProperty;
                }
            }
        } elseif ($object->$property) {
            switch ($property) {
                case 'sqlTable':
                    $this->$property = new \IMIA\BaseExt\Annotation\SQL\Table();
                    $this->mergeObjectProperty($object, $property);
                    break;
                case 'tcaTable':
                    $this->$property = new \IMIA\BaseExt\Annotation\TCA\Table();
                    $this->mergeObjectProperty($object, $property);
                    break;
                case 'tcaAddPaletteToTypes':
                    $this->$property = new \IMIA\BaseExt\Annotation\TCA\AddPaletteToTypes();
                    $this->mergeObjectProperty($object, $property);
                    break;
                case 'tcaAddPalettesToTypes':
                    $this->$property = new \IMIA\BaseExt\Annotation\TCA\AddPalettesToTypes();
                    $this->mergeObjectProperty($object, $property);
                    break;
                default:
                    $this->$property = $object->$property;
                    break;
            }
        }
    }
}
