<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Utility;

/**
 * @package     imia_base_ext
 * @subpackage  Utility
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class Flexform
{
    /**
     * @var string
     */
    protected $extensionKey = 'tx_imiabaseext';

    /**
     * @var array
     */
    private $pluginSettings;

    /**
     * @var \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
     */
    private $contentObjectRenderer;

    /**
     * @param array $params
     * @param array $items
     */
    protected function addItems(&$params, $items)
    {
        if (is_array($items)) {
            foreach ($items as $item) {
                $label = $this->getContentObjectRenderer()->stdWrap($item['label'], $item['label.']);
                $value = $this->getContentObjectRenderer()->stdWrap($item['value'], $item['value.']);

                if (preg_match('/^(EXT:)?([^\/]+)\//ism', $value, $match)) {
                    $label = '[' . $match[2] . '] ' . $label;
                }

                $params['items'][] = array(
                    $label,
                    $value,
                    null
                );
            }
        }
    }

    /**
     * @param string $name
     * @param integer $pageUid
     * @return array
     */
    protected function getPluginSettings($name, $pageUid)
    {
        if (!$this->pluginSettings) {
            $typoScript = BackendTypoScript::get($pageUid);
            $this->pluginSettings = $typoScript['plugin.'][$this->extensionKey . '.']['flexforms.'];
        }

        if (array_key_exists($name, $this->pluginSettings)) {
            return $this->pluginSettings[$name];
        } else {
            return null;
        }
    }

    /**
     * @return \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
     */
    protected function getContentObjectRenderer()
    {
        if (!$this->contentObjectRenderer) {
            $objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager');
            $GLOBALS['TSFE'] = $objectManager->create('TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController', $GLOBALS['TYPO3_CONF_VARS'], 0, 0);
            $GLOBALS['TSFE']->config['config']['language'] = $GLOBALS['LANG']->lang;
            $GLOBALS['TSFE']->initLLvars();
            $GLOBALS['TSFE']->initTemplate();

            $this->contentObjectRenderer = $objectManager->create('TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer');
        }

        return $this->contentObjectRenderer;
    }
}