<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Utility;

/**
 * @package     imia_base_ext
 * @subpackage  Utility
 * @author      David Frerich <d.frerich@imia.de>
 */
class Helper
{
    /**
     * @param \TYPO3\CMS\Core\Resource\File|integer $file
     * @param array $fileReferenceData
     * @param array $insertData
     * @return \TYPO3\CMS\Core\Resource\FileReference
     */
    static public function createFileReference($file, array $fileReferenceData = array(), $insertData = null)
    {
        $objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager');
        $factory = $objectManager->get('TYPO3\CMS\Core\Resource\ResourceFactory');
        if (!is_object($file)) {
            try {
                $file = $factory->getFileObject((int)$file);
            } catch(\Exception $e) { }
        }

        $fileReference = null;
        if ($file && is_object($file)) {
            $fileReferenceData['uid_local'] = $file->getUid();

            /** @FIXME currently there is no other way to insert a new file reference */
            if ($insertData && $insertData['tablenames'] && $insertData['fieldname'] && $insertData['uid_foreign']) {
                $result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('sorting', 'sys_file_reference',
                    'pid = ' . (int)$insertData['pid'] . ' AND uid_foreign = ' . $insertData['uid_foreign'] .
                    ' AND tablenames = "' . addslashes($insertData['tablenames']) . '" AND fieldname = "' . $insertData['fieldname'] . '"' .
                    ' AND table_local = "sys_file"', '', 'sorting DESC', 1);
                $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result);

                $GLOBALS['TYPO3_DB']->exec_INSERTquery('sys_file_reference', array_merge(array(
                    'pid'           => (int)$insertData['pid'],
                    'uid_foreign'   => (int)$insertData['uid_foreign'],
                    'tablenames'    => $insertData['tablenames'],
                    'fieldname'     => $insertData['fieldname'],
                    'table_local'   => 'sys_file',
                    'crdate'        => time(),
                    'tstamp'        => time(),
                    'sorting'       => $row && $row['sorting'] ? (int)$row['sorting'] + 1 : 1,
                ), $fileReferenceData));

                $fileReferenceData['uid'] =  $GLOBALS['TYPO3_DB']->sql_insert_id();
            }
            $fileReference = $factory->createFileReferenceObject($fileReferenceData);
        }

        return $fileReference;
    }

    /**
     * @param \TYPO3\CMS\Core\Resource\FileReference $fileReference
     */
    static public function deleteFileReference(\TYPO3\CMS\Core\Resource\FileReference $fileReference)
    {
        // delete file
        if (file_exists(PATH_site . $fileReference->getPublicUrl())) {
            $fileReference->getOriginalFile()->delete();
        }

        // delete file reference
        /** @FIXME typo3 6.0.x #not yet implemented, but prepared in api
        $imageFile->delete() */
        $GLOBALS['TYPO3_DB']->exec_DELETEquery('sys_file_reference', 'uid = ' . $fileReference->getUid());
    }

    /**
     * @param \TYPO3\CMS\Core\Resource\FileReference $fileReference
     * @param \TYPO3\CMS\Core\Resource\File $file
     * @param boolean $deleteOldFile
     * @param integer $sorting
     * @return \TYPO3\CMS\Core\Resource\FileReference
     */
    static public function updateFileReference(\TYPO3\CMS\Core\Resource\FileReference &$fileReference,
        \TYPO3\CMS\Core\Resource\File $file = null, $deleteOldFile = true, $sorting = null)
    {
        $oldFile = $fileReference->getOriginalFile();

        $mergedFileReferenceData = $fileReference->toArray();
        $fileReferenceData = array(
            'pid'           => $mergedFileReferenceData['pid'],
            'tstamp'        => time(),
            'crdate'        => $mergedFileReferenceData['crdate'],
            'title'         => $mergedFileReferenceData['title'],
            'description'   => $mergedFileReferenceData['description'],
            'alternative'   => $mergedFileReferenceData['alternative'],
            'link'          => $mergedFileReferenceData['link'],
            'tablenames'    => $mergedFileReferenceData['tablenames'],
            'fieldname'     => $mergedFileReferenceData['fieldname'],
            'table_local'   => $mergedFileReferenceData['table_local'],
        );

        if ($file) {
            $fileReferenceData['uid_local'] = $file->getUid();
        }
        if ($sorting !== null) {
            $fileReferenceData['sorting_foreign'] = (int)$sorting;
        }

        $GLOBALS['TYPO3_DB']->exec_UPDATEquery(
            'sys_file_reference',
            'uid = ' . (int)$mergedFileReferenceData['uid'],
            $fileReferenceData
        );

        if ($file) {
            $fileReferenceClass = get_class($fileReference);
            $fileReference = new $fileReferenceClass($fileReferenceData);
        }

        if ($deleteOldFile && $file) {
            $oldFile->delete();
        }

        return $fileReference;
    }

    /**
     * @param array $objects
     * @param string $property
     * @param string $direction
     */
    static public function sortObjectArrayByProperty($objects, $property, $direction = 'asc')
    {
        if (!is_array($objects)) {
            $objectArray = array();
            foreach ($objects as $object) {
                $objectArray[] = $object;
            }

            $objects = $objectArray;
        }

        $propertyMethod = 'get' . ucfirst($property);
        usort($objects, function($a, $b) use($propertyMethod) {
            $propertyValueA = Helper::getOptimizedValue($a->$propertyMethod());
            $propertyValueB = Helper::getOptimizedValue($b->$propertyMethod());

            $compare = strcmp($propertyValueA, $propertyValueB);

            return $compare;
        });

        if (strtolower($direction) != 'asc') {
            $objects = array_reverse($objects);
        }

        return $objects;
    }

    /**
     * @param array $objects
     * @param array $properties
     */
    static public function sortObjectArrayByProperties($objects, $properties)
    {
        if (!is_array($objects)) {
            $objectArray = array();
            foreach ($objects as $object) {
                $objectArray[] = $object;
            }

            $objects = $objectArray;
        }

        $propertyMethods = array();
        $propertyDirections = array();
        foreach (array_reverse($properties) as $property => $direction) {
            $propertyMethods[] = 'get' . ucfirst($property);
            $propertyDirections[] = strtolower($direction);
        }

        usort($objects, function($a, $b) use($propertyMethods, $propertyDirections, $helper) {
            $compare = 0;
            foreach ($propertyMethods as $key => $propertyMethod) {
                $propertyValueA = Helper::getOptimizedValue($a->$propertyMethod());
                $propertyValueB = Helper::getOptimizedValue($b->$propertyMethod());

                $compare = strcmp($propertyValueA, $propertyValueB);
                if ($propertyDirections[$key] != 'asc') {
                    $compare *= -1;
                }

                if ($compare != 0) {
                    break;
                }
            }

            return $compare;
        });

        return $objects;
    }

    static public function getOptimizedValue($value)
    {
        if (is_object($value) && get_class($value) == 'DateTime') {
            $value = $value->getTimestamp();
        }

        return $value;
    }
}